#!/bin/sh
# FILE: loadtest.tcl
#
# This script runs one or more boxsi jobs in parallel and reports the
# total run time.  The first parameter is the number of jobs to run in
# parallel.  The remaining parameters are MIF 2.x files to run
#
#    Bootstrap trick \
exec tclsh "$0" ${1+"$@"}

# Set these variables appropriately
set OommfRootDir ..
set TclCmd tclsh

set OommfCmd [file join $OommfRootDir oommf.tcl]
set JobTemplate [list $TclCmd $OommfCmd boxsi -kill all %s]

proc Usage {} {
    puts stderr {Usage: loadtest <parallel count> job1.mif [job2.mif ...]}
    exit 1
}

if {[llength $argv]<2 \
	|| [string match [lindex $argv 0] "-h"] \
	|| [string match [lindex $argv 0] "--help"] } {
    Usage
}

set parallel_count [lindex $argv 0]
set joblist [lrange $argv 1 end]

if {[package vsatisfies [package provide Tcl] 8.3]} {
    set time_milliseconds 1
} else {
    set time_milliseconds 0
}

proc ReadTest { id chan } {
    if {[eof $chan]} {
	if {[catch {close $chan} errcode]} {
	    puts [format "ERROR %2d: $errcode" $id]
	} else {
	    puts [format "%3d END" $id]
	}
	global donecount
	incr donecount
    } else {
	while {[gets $chan text]>=0} {
	    puts [format "%3d-> $text" $id]
	}
    }
}

set timefmt "%a %b %d %H:%M:%S %Z %Y"
puts "JOB COUNT: [llength $joblist]"
puts "START TIME: [clock format [clock seconds] -format $timefmt]"
if {$time_milliseconds} {
 set timestart [clock clicks -milliseconds]
} else {
 set timestart [clock seconds]
}

set launchcount 0
set donecount 0
while {$launchcount<[llength $joblist]} {
    if {$launchcount-$donecount>=$parallel_count} {
	# All compute threads busy
	vwait donecount
    }
    set job [format $JobTemplate [lindex $joblist $launchcount]]
    incr launchcount
    puts [format "CMD%3d: $job" $launchcount]
    set chan [open "| $job 2>@ stdout"]
    fconfigure $chan -blocking 0
    fileevent $chan readable [list ReadTest $launchcount $chan]
}

while {$donecount<$launchcount} {
    vwait donecount ;# Wait for all jobs to finish
}

if {$time_milliseconds} {
 set timestop [clock clicks -milliseconds]
} else {
 set timestop [clock seconds]
}
puts "STOP TIME: [clock format [clock seconds] -format $timefmt]"
if {$time_milliseconds} {
   set timediff [expr {($timestop-$timestart)*0.001}]
   puts [format "ELAPSED TIME: %.3f s" $timediff]
} else {
   set timediff [expr {($timestop-$timestart)}]
   puts [format "ELAPSED TIME: %.0f s" $timediff]
}
